@@ -38,10 +38,12 @@ impl<T> WithLock<T> {
 	/// use std::sync::Mutex;
 	/// use with_lock::WithLock;
 	///
-	/// let a = WithLock::<i64>::new(Mutex::new(2));
-	/// let b = WithLock::<i64>::new(Mutex::new(3));
-	/// let a_lock = a.with_lock(|s| *s);
-	/// let b_lock = b.with_lock(|s| *s);
+	/// let a = WithLock::<i64>::new(2);
+	/// let b = WithLock::<i64>::new(3);
+	/// let action_and_get = |s: &mut i64| *s;
+	/// let a_lock = a.with_lock(action_and_get);
+	/// let b_lock = b.with_lock(action_and_get);
+
 	/// assert_eq!(a_lock + b_lock, 5);
 	/// let a_lock_2 = a.with_lock(|s| *s);
 	/// let b_lock_2 = b.with_lock(|s| *s);
@@ -60,10 +62,12 @@ impl<T> WithLock<T> {
 	/// ## Examples
 	/// ```rust
 	/// use with_lock::WithLock;
-	/// WithLock::<i64>::new(std::sync::Mutex::new(123));
+	/// WithLock::<i64>::new(123);
 	/// ```
-	pub fn new<F>(data: Mutex<F>) -> WithLock<F> {
-		WithLock { data }
+	pub fn new<F>(data: F) -> WithLock<F> {
+		WithLock {
+			data: Mutex::new(data),
+		}
 	}
 }
 
@@ -88,7 +92,7 @@ impl<T> MutexCell<T> {
 	/// ```
 	pub fn new(data: T) -> MutexCell<T> {
 		MutexCell {
-			data: WithLock::<T>::new(Mutex::new(data)),
+			data: WithLock::<T>::new(data),
 		}
 	}
 
@@ -164,6 +168,43 @@ mod readme {}
 mod tests {
 	use crate::*;
 
+	struct SharedData {
+		pub a: i64,
+		pub b: i64,
+	}
+
+	#[test]
+	fn test_with_lock() {
+		let a = WithLock::<i64>::new(2);
+		let b = WithLock::<i64>::new(3);
+
+		let action_and_get = |s: &mut i64| *s;
+		let a_lock = a.with_lock(action_and_get);
+		let b_lock = b.with_lock(action_and_get);
+		assert_eq!(a_lock + b_lock, 5);
+
+		// repeat action with embedded lambda expression
+		let a_lock_2 = a.with_lock(|s| *s);
+		let b_lock_2 = b.with_lock(|s| *s);
+		assert_eq!(a_lock_2 + b_lock_2, 5);
+	}
+
+	#[test]
+	fn test_with_lock_over_struct() {
+		let a = WithLock::<SharedData>::new(SharedData { a: 2, b: 2 });
+		let b = WithLock::<SharedData>::new(SharedData { a: 3, b: 3 });
+
+		let action_and_get = |s: &mut SharedData| (*s).a;
+		let a_lock = a.with_lock(action_and_get);
+		let b_lock = b.with_lock(action_and_get);
+		assert_eq!(a_lock + b_lock, 5);
+
+		// repeat action with embedded lambda expression and member b (avoid dead code warning)
+		let a_lock_2 = a.with_lock(|s| (*s).b);
+		let b_lock_2 = b.with_lock(|s| (*s).b);
+		assert_eq!(a_lock_2 + b_lock_2, 5);
+	}
+
 	#[test]
 	fn test_mutex_cell_no_deadlocks() {
 		let a = MutexCell::new(2);
